<?php
/*
 * ToFeed - Non-intrusive RSS feed generator for MediaWiki articles.
 * Copyright (C) 2009 Sylvain Leroux - sylvain@chicoree.fr
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
require_once("$IP/includes/Feed.php");

/*
 * Extension infos
 *
 * History:
 * 1.0b	(2009-11-19)	First published version
 * 1.1b	(2010-02-28)	Fixed an issue with mimetype preventing
 *                      to read feeds with Internet Explorer
 * 1.2b (2010-11-30)    Add support to display articles in category as feed
 * 1.2c (2012-05-17)	Fixed an issue where some version of libxml2
 *			complains about "ID <anchor> already defined in Entity"
 * 1.2d (2012-06-07)	Fixed a strict warning message complaining that
 *			"Non-static method DOMDocument::load() should not 
 *                       be called statically"
 */
$wgExtensionCredits['other'][] = array(
    'name' => 'ToFeed',
    'version' => '1.2d',
    'author' => 'Sylvain Leroux <sylvain@chicoree.fr>',
    'url' => 'http://www.chicoree.fr',
    'description' => 'Add a new action to retrieve an article as a feed',
    'descriptionmsg' => '',
);

/**
 * Add feed link on each page
 */
$wgHooks['BeforePageDisplay'][] = 'noFeed';

function noFeed(&$output) {
	$output->setSyndicated(false); // this doen't remove the 'recent changes' feed :(
	$output->addLink(array("rel"=>"alternate", "type"=>"application/rss+xml", "title"=>"Flux RSS de cette page","href"=>"?action=tofeed"));
	
	return true;
}

/**
 * Hook to UnknownAction
 */
$wgHooks['UnknownAction'][]='toFeedAction';

/**
 * Convert the page to a feed.
 */
function toFeedAction($action,$article) {
	// If it is not for ourselves, pass
    if ($action != "tofeed")
        return true;

    global $wgOut;
    global $wgSitename;
    global $tfArticle;
    
    $tfArticle = $article;

    /* Prepare wgOut to output a RSS feed and nothing else. */
    $mimetype = "application/xml";
    header( "Content-type: application/xml; charset=UTF-8" );
    $wgOut->sendCacheControl();
    $wgOut->disable();
    
    $content = $wgOut->parse($article->getContent());   


    // The mediawiki parser add both a "name" and an "id" attribute
    // to the generated anchors, but some versions of libxml2
    // reports that as a warning
    $search  = '/<a\s+name="([^"]*)"\s+id="\1"/';
    $replace = '<a name="\1"';
    $content = preg_replace($search, $replace, $content);

//    print_r($article);
    $title = $article->mTitle;
    
    // Build a HTML document containing our parsed article.
    // This document will serve as source document for XSLT.
    $header[] = "<meta http-equiv='content-type' content='text/html; charset=UTF-8' />";
    $header[] = "<title>" . $article->mTitle->getText() . " - $wgSitename</title>";
    $header[] = "<base href='" . $article->mTitle->getFullUrl() . "' />";
    
    // For category pages, add members
    if (NS_CATEGORY == $article->mTitle->getNamespace()) {
        $category = Category::newFromTitle($article->mTitle);
        $members = $category->getMembers();
        
        $content .= "<ul>";
        foreach($members as $member) {
            $url = $member->getFullUrl();
            $text = $member->getText();
            $content .= "<li><a href='$url'>$text</a></li>";
//            print_r($member);
        }
        $content .= "</ul>";
    }
    
    $htmlDocument = "<html><head>" . join("\n", $header) . "</head><body>$content</body></html>";
    
    // echo ($htmlDocument);
    echo htmlDocumentToFeed($htmlDocument);

//    /* debug */
//    print("<!--\n");
//    print_r($content);
//    print("\n-->");
    
    return false; // do not execute other action
}

/**
 * Build an absolute URL.
 * 
 * MediaWiki produce only:
 * - full URL: http://www.chicoree.fr/something
 * - absolute URLs: /w/Accueil
 * - fragment: #top
 * 
 * @param $base URL of the base page. *Must* be a full URL.
 * @param $url The 'local' URL. Could be a full URL, an absolute URL or a fragment.
 *              Currently does not supports *relative* URL.
 * @return unknown_type
 */
function tfResolveFullURL($base, $url) {
    // Absolute URL
    if (strlen($url) > 0 && $url[0] == '/') {
        $r = array();
        preg_match("/^[a-z]+:\/*[^\/]*/", $base, $r);
        
        return $r[0] . $url;
    }

    // Fragment
    if (strlen($url) > 0 && $url[0] == '#') {
        return $base . $url;
    }
    
	// Full URL
	if (preg_match("/^[a-z]+:/", $url))
	   return $url;
	
    
	// Otherwise assume relative URL
	if (preg_match("/^([a-z]+:\/*[^\/]+(\/[^\/]*)*)\/[^\/]*/", $base, $r)) {
        $base = $r[1];
//        print_r($r);
	}
    return $base . '/' . $url;
}

function tfGetArticleAuthor() {
    global $tfArticle;
    
    if ($tfArticle->mRevision)
        return $tfArticle->mRevision->mUserText;
    
    return null;
}

function tfGetRevisionId() {
    global $tfArticle;
    
    if ($tfArticle->mRevision)
        return $tfArticle->mRevision->mId;
    
    return null;
}

/**
 * Convert an HTML document to a feed.
 * 
 * Process the document throught XSLT in order to produce an RSS feed.
 * Returns the feed as a string.
 */
function htmlDocumentToFeed($htmlDocument) {
	global $IP;

	$document = new DOMDocument();	
	$document->loadHTML($htmlDocument);
	
//	echo $document->saveXML();

	$stylesheet = new DOMDocument();	
	$stylesheet->load("$IP/extensions/ToFeed/RSS.xsl");

	$processor  = new XSLTProcessor();
	$processor->registerPHPFunctions('tfResolveFullURL');
	$processor->importStylesheet($stylesheet);
		
	return $processor->transformToXml($document);
}
